function [H, sigma2, logl] = dcv_em(X, v, H_0, sigma2_0, Nit);

%dcv_em    Blind identification using the EM algorithm.
%Use:	[H,sigma2,logl] = dcv_em(X,v,H_0,sigma2_0,Nit). Returns the estimated
%	quantities corresponding to each iteration (ie. H(:,n), sigma2(n),
%	logl(n) for n = 1 (initialization) to n = Nit+1 (last iteration).

% DCV Toolbox, version 1.1
% Olivier Capp, 24/12/97 - 24/12/97
% ENST Dpt. Signal / CNRS URA 820, Paris

% Input args.
error(nargchk(5, 5, nargin));
% Set parameters for the DCV functions
[AR,AC,V]=dcv_init(v,length(H_0));
[NStat,NSymb,LFilt,REAL_COEFF]=dcv_chk(AR,AC,V,H_0);

% Estimate parameters using EM
H = zeros(LFilt, Nit+1);
sigma2 = zeros(1, Nit+1);
logl = zeros(1, Nit+1);
% Store initalization values
H(:,1) = H_0;
sigma2(1) = sigma2_0;

fprintf(1,'Iteration:');
for n=1:Nit
  fprintf(1, ' %d', n);
  [logl(n), gamma] = dcv_fb(X,AR,AC,V,H(:,n),sigma2(n));
  [H(:,n+1),sigma2(n+1)] = dcv_par(X,gamma,V);
end;
logl(Nit+1) = dcv_fb(X,AR,AC,V,H(:,Nit+1),sigma2(Nit+1));
fprintf(1,'\n');
