function [AR, AC, V] = dcv_init(v, l);

%dcv_init  Returns transition matrix and state vectors for a convolution
%          model.	
%	Use: [AR,AC,V]=dcv_init(v,l). v are the discrete values (should
%	be distinct), l is the length of the filter. AR and AC respectively
%	contain the indexes corresponding to non null probabilities for each
%	row and each column of the transition matrix. V are the state vectors
%	corresponding to each state (stacked as column vectors). 

% DCV Toolbox, version 1.1
% Olivier Capp, 23/07/96 - 31/12/98
% ENST Dpt. Signal / CNRS URA 820, Paris

% Input args.
error(nargchk(2, 2, nargin));
% Number of discrete values
N = length(v);

% Construct all possible state vectors
V = zeros(l,  N^l);
for li = 0:((N^l)-1)
  tmp = li;
  for ik = l-1:-1:0
    ni = fix(tmp/(N^ik));
    tmp = tmp-ni*(N^ik);
    V(l-ik, 1+li) = v(1+ni);
  end;
end;

% Obtain the non nul indexes for each row of the transition matrix
% (the way it is done is not smart at all, but it works)
AR = zeros(N^l, N);
for i = 1:N^l
  if (l > 2)
    ind = find(all(V(1:l-1,i)*ones(1,N^l) == V(2:l,:)));
  elseif (l == 2)
    ind = find(V(1:l-1,i)*ones(1,N^l) == V(2:l,:));
  elseif (l == 1)
    ind = 1:N;
  else
    error('Filter length should be at least 1.');
  end;
  AR(i,:) = ind;
end;
% Do the same for the columns of the transition matrix
AC = zeros(N, N^l);
for i = 1:N^l
  if (l > 2)
    ind = find(all(V(2:l,i)*ones(1,N^l) == V(1:l-1,:)));
  elseif (l == 2)
    ind = find(V(2:l,i)*ones(1,N^l) == V(1:l-1,:));
  elseif (l == 1)
    ind = 1:N;
  end;
  AC(:,i) = ind';
end;
