function [f,g] = dcv_qn_f(args, X, AR, AC, V, VAR_OFFSET, REAL_COEFF);

%dcv_qn_f  Subfunction of dcv_qn (computes the oposite of the log-likelihood)
%          and associated gradient.
%Use:	[f,g]=dcv_qn_f(args,X,AR,AC,V,VAR_OFFSET, REAL_COEFF);

% DCV Toolbox, $Revision: 1.2 $
% Olivier Capp, 10/03/98 - $Date: 2004/11/19 10:58:45 $
% http://www.tsi.enst.fr/~cappe/dcv/

% Get input parameters
l = length(args);
if (REAL_COEFF)
  H = args(1:l-1);
else
  % Real and imaginary parts are passed in two different arrays because fminunc
  % does not support complex arguments
  H = args(1:(l-1)/2) + i*args(1+(l-1)/2:l-1);
end;
sigma2 = exp(args(l));

% Call the computational function (use a minus sign because fminunc needs a
% function to be minimized)
f = -dcv_llk (X, AR, AC, V, H, sigma2, VAR_OFFSET);

% Warning concerning underflows occurring in dcv_llk are ignored because they
% usually have no consequences (at least if it only happens once). If the
% situation gets real bad, warnings will occur in dcv_qn_g.

% Call the gradient computation
[Dlogl_H, Dlogl_logvar,nundflw] = dcv_gllk(X, AR, AC, V, H, sigma2, VAR_OFFSET);
% Echo a warning in case of underflows (usually the optimization will fail if
% it gets to this point)
if (nundflw > 0)
  fprintf(1,'Warning: %d underflows in dcv_gllk.\n', nundflw);
end;

% Ouput argument (the oposite of the log-likelihood is the function to be
% minized)
if (REAL_COEFF)
  g = -[Dlogl_H; (Dlogl_logvar)];
else
  g = -[real(Dlogl_H); imag(Dlogl_H); (Dlogl_logvar)];
end
