function [symb, State1, logl] = dcv_vit(X, v, H, sigma2);

%dcv_vit   Computes the most likely sequence of symbols and initial state
%	   (using "Viterbi" dynamic programming).
%	Use: [symb,State1,logl] = dcv_vit(X,v,H,sigma2).

% DCV Toolbox, version 1.1
% Olivier Capp, 29/12/97 - 29/12/97
% ENST Dpt. Signal / CNRS URA 820, Paris

% Input arguments
error(nargchk(4, 4, nargin));
[AR,AC,V]=dcv_init(v,length(H));
[NStat, NSymb, LFilt, REAL_COEFF] = dcv_chk(AR,AC,V,H);
[T, Nc] = size(X);
if (Nc ~= 1)
  error('Observations should be specified as a row vector.');
end;

% Compute density values on the log scale omitting common normalization factors
logdens = zeros(T, NStat);
if (REAL_COEFF)
  for n = 1:NStat
    mu = H'*V(:,n);
    logdens(:,n) = -((X-mu).^2);
  end;
else
  % Beware: In this case, sigma2 is the power of the noise which is
  % assumed to be complex and circular
  for n = 1:NStat
    mu = H'*V(:,n);
    logdens(:,n) = -real((X-mu).*conj(X-mu));
  end;
end;

% Viterbi recursion
% Transition probabilities and initial probabilities can be omitted
% since they are equals 
% Compute partial loglikelihood array and bactracking array
plogl = -Inf*ones(T, NStat);
bcktr = zeros(T-1, NStat);
plogl(1,:) = logdens(1,:);
for t = 2:T
  for n = 1:NStat
    [plogl(t,n), ind] = max(plogl(t-1, AC(:,n)));
    plogl(t,n) = plogl(t,n) + logdens(t,n);
    bcktr(t-1,n) = AC(ind,n);
  end;
end;

% Backtracking
path = zeros(T,1);
[logl, path(T)] = max(plogl(T,:));
for t=(T-1):-1:1
  path(t) = bcktr(t, path(t+1));
end;

% Symbols and initial state
State1 = V(:,path(1));
symb = V(1,path).';

% Log-likelihood with proper normalization
if (REAL_COEFF)
  logl = 0.5*logl/sigma2 - (T/2)*log(2*pi*sigma2) - (T-1)*log(NSymb) - log(NStat);
else
  logl = logl/sigma2 - T*log(pi*sigma2) - (T-1)*log(NSymb) - log(NStat);
end;
